package com.cisco.ass.distr;

import com.cisco.ass.distr.tools.Factorization;
import com.cisco.ass.distr.tools.NumberSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;

public class CallableImpl implements Test {

    private static final Logger LOG = LoggerFactory.getLogger(CallableImpl.class);

    private static final int THREAD_COUNT = 8;

    @Override
    public void test(NumberSource source, Factorization factorization) throws Exception {
        ExecutorService executorService = Executors.newFixedThreadPool(THREAD_COUNT);
        List<Future<List<BigInteger>>> futures = new ArrayList<>();

        for (BigInteger n : source) {
            futures.add(executorService.submit(new FactorizationCallable(n, factorization)));
//            futures.add(executorService.submit(() -> factorization.factor(n))); // this is equivalent to the previous line using lambda function (without the logging)
        }

        LOG.info("All callables submitted");

        for (Future<List<BigInteger>> future : futures) {
            LOG.info(">"+future.get());
        }


        LOG.info("Calling shutdown");
        executorService.shutdown();
        LOG.info("Shutdown succeeded");
        try {
            executorService.awaitTermination(1, TimeUnit.HOURS);
        } catch (InterruptedException ex) {
            throw new RuntimeException(ex);
        }
        LOG.info("Completed");
    }

    private static class FactorizationCallable implements Callable<List<BigInteger>> {

        private final BigInteger number;
        private final Factorization factorization;

        private FactorizationCallable(final BigInteger number, final Factorization factorization) {
            this.number = number;
            this.factorization = factorization;
        }

        @Override
        public List<BigInteger> call() throws Exception {
            List<BigInteger> factors = factorization.factor(number);
            LOG.info("Factors: {}", factors);
            return factors;
        }
    }
}
