package com.cisco.ass.distr;

import com.cisco.ass.distr.tools.Factorization;
import com.cisco.ass.distr.tools.NumberSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import rx.Observable;
import rx.schedulers.Schedulers;

import java.math.BigInteger;
import java.util.AbstractMap;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class RxImpl implements Test {

    private static final Logger LOG = LoggerFactory.getLogger(RxImpl.class);

    private static final int THREAD_COUNT = 8;

    @Override
    public void test(NumberSource source, Factorization factorization) throws Exception {
        Observable<BigInteger> inputNumbers = Observable.from(source);

        ExecutorService es = Executors.newFixedThreadPool(THREAD_COUNT);

        inputNumbers
                .flatMap(n -> Observable.just(n).observeOn(Schedulers.from(es)).map(factorization::factor))
                .doOnNext(x -> LOG.info("Factors: {}", x))
                .flatMap(this::getFactorsFormatted)
                .subscribe(x -> LOG.info("Formatted: {}", x));

        es.shutdown();
        es.awaitTermination(1, TimeUnit.HOURS);
    }

    private Observable<String> getFactorsFormatted(List<BigInteger> factors) {
        return Observable.from(factors)
                .groupBy(f -> f)
                .flatMap(x -> Observable.just(x.getKey()).zipWith(x.count(), AbstractMap.SimpleEntry::new))
                .sorted((e1,e2) -> e1.getKey().compareTo(e2.getKey()))
                .map(x -> formatFactorWithPower(x.getKey(), x.getValue()))
                .collect(StringBuilder::new, this::add) // uses mutable data structure
                .map(StringBuilder::toString);
//                .reduce((in,s) -> in+" * "+s); // this gives the same result as the previous collect+map, but not using a mutable data structure
    }

    private String formatFactorWithPower(BigInteger factor, Integer power) {
        if (power.equals(1)) {
            return factor.toString();
        } else {
            return String.format("%s^%s", factor, power);
        }
    }

    private void add(StringBuilder sb, String s) {
        if (sb.length() == 0) {
            sb.append(s);
        } else {
            sb.append(" * ").append(s);
        }
    }
}
