package com.cisco.ass.distr.tools;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.LineIterator;

import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.util.Iterator;

public class FileNumberSource implements NumberSource {

    private final File file;

    public FileNumberSource(File file) {
        this.file = file;
    }

    @Override
    public Iterator<BigInteger> iterator() {
        try {
            return new FileNumberIterator();
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }

    private class FileNumberIterator implements Iterator<BigInteger>, AutoCloseable {

        private final LineIterator iterator;

        FileNumberIterator() throws IOException {
            iterator = FileUtils.lineIterator(file, StandardCharsets.UTF_8.name());
        }

        @Override
        public void close() throws Exception {
            iterator.close();
        }

        @Override
        public boolean hasNext() {
            return iterator.hasNext();
        };

        @Override
        public BigInteger next() {
            return new BigInteger(iterator.next());
        }
    }

}
